@@ -381,36 +381,70 @@ class Net_IMAP extends Net_IMAPProtocol {
             $message_set="1:*";
         }
         if($uidFetch) {
-          $ret=$this->cmdUidFetch($message_set,"(RFC822.SIZE UID FLAGS ENVELOPE INTERNALDATE BODY.PEEK[HEADER.FIELDS (CONTENT-TYPE)])");
+		  #error_log("egw-pear::NET::IMAP:getSummary->fetch by UID ".$message_set);
+          $ret=$this->cmdUidFetch($message_set,"(RFC822.SIZE UID FLAGS ENVELOPE INTERNALDATE BODY.PEEK[HEADER.FIELDS (CONTENT-TYPE X-PRIORITY)])");
         } else {
-          $ret=$this->cmdFetch($message_set,"(RFC822.SIZE UID FLAGS ENVELOPE INTERNALDATE BODY.PEEK[HEADER.FIELDS (CONTENT-TYPE)])");
+		  #error_log("egw-pear::NET::IMAP:getSummary->fetch message ".$message_set);
+          $ret=$this->cmdFetch($message_set,"(RFC822.SIZE UID FLAGS ENVELOPE INTERNALDATE BODY.PEEK[HEADER.FIELDS (CONTENT-TYPE X-PRIORITY)])");
         }
+		#error_log(print_r($ret['PARSED'][0],true));
         #$ret=$this->cmdFetch($message_set,"(RFC822.SIZE UID FLAGS ENVELOPE INTERNALDATE BODY[1.MIME])");
-        if (PEAR::isError($ret)) {
-            return $ret;
-        }
-        if(strtoupper($ret["RESPONSE"]["CODE"]) != "OK"){
-            return new PEAR_Error($ret["RESPONSE"]["CODE"] . ", " . $ret["RESPONSE"]["STR_CODE"]);
+        if (PEAR::isError($ret) || strtoupper($ret["RESPONSE"]["CODE"]) != "OK") {
+			error_log("egw-pear::NET::IMAP:getSummary->error after Fetch for message(s):".$message_set." Trying to retrieve single messages.");
+			unset($ret);
+			# if there is an error, while retrieving the information for the whole list, try to retrieve the info one by one, to be more error tolerant
+			foreach (explode(',',$message_set) as $msgid) {
+				$retloop=$this->cmdUidFetch($msgid,"(RFC822.SIZE UID FLAGS ENVELOPE INTERNALDATE BODY.PEEK[HEADER.FIELDS (CONTENT-TYPE X-PRIORITY)])");
+				if (PEAR::isError($retloop)|| strtoupper($retloop["RESPONSE"]["CODE"]) != "OK") {
+					# log the error, and create a dummy-message as placeholder, this may hold the possibility to read the message anyway
+					error_log("egw-pear::NET::IMAP:getSummary->error after Fetch for message with id:".$msgid);
+					error_log($ret["RESPONSE"]["CODE"] . ", " . $ret["RESPONSE"]["STR_CODE"]);
+					$ret['PARSED'][]=array('COMMAND'=>"FETCH",'EXT'=>array('UID'=>$msgid,'ENVELOPE'=>array('SUBJECT'=>"[FELAMIMAIL:ERROR]can not parse this message(header).",)));
+				} else {
+					#error_log(print_r($retloop['PARSED'][0],true));
+					# renew the response for every message retrieved, since the returnvalue is structured that way
+					$ret['RESPONSE']=$retloop['RESPONSE'];
+					$ret['PARSED'][]=$retloop['PARSED'][0];
+				}
+				unset($retloop);
+			}
+            #return $ret;
         }
+		# this seems to be obsolet, since errors while retrieving header informations are 'covered' above
+        #if(strtoupper($ret["RESPONSE"]["CODE"]) != "OK"){
+		#	error_log("egw-pear::NET::IMAP:getSummary->ResponseCode not OK");
+        #    return new PEAR_Error($ret["RESPONSE"]["CODE"] . ", " . $ret["RESPONSE"]["STR_CODE"]);
+        #}
 
-        #print "<hr>"; var_dump($ret["PARSED"]); print "<hr>";
+        #print "<hr>"; 
+		#error_log("egw-pear::NET::IMAP:getSummary->".print_r($ret["PARSED"],TRUE)); 
+		#print "<hr>";
 
         if(isset( $ret["PARSED"] ) ){
             for($i=0; $i<count($ret["PARSED"]) ; $i++){
+				if ($ret["PARSED"][$i]['COMMAND'] != 'FETCH') continue;
                 $a=$ret["PARSED"][$i]['EXT']['ENVELOPE'];
                 $a['MSG_NUM']=$ret["PARSED"][$i]['NRO'];
                 $a['UID']=$ret["PARSED"][$i]['EXT']['UID'];
                 $a['FLAGS']=$ret["PARSED"][$i]['EXT']['FLAGS'];
                 $a['INTERNALDATE']=$ret["PARSED"][$i]['EXT']['INTERNALDATE'];
                 $a['SIZE']=$ret["PARSED"][$i]['EXT']['RFC822.SIZE'];
-                if(isset($ret["PARSED"][$i]['EXT']['BODY[HEADER.FIELDS (CONTENT-TYPE)]']['CONTENT'])) {
-                    if(preg_match('/^content-type: (.*);/iU', $ret["PARSED"][$i]['EXT']['BODY[HEADER.FIELDS (CONTENT-TYPE)]']['CONTENT'], $matches)) {
+                if(isset($ret["PARSED"][$i]['EXT']['BODY[HEADER.FIELDS (CONTENT-TYPE X-PRIORITY)]']['CONTENT'])) {
+                    if(preg_match('/content-type: (.*);/iU', $ret["PARSED"][$i]['EXT']['BODY[HEADER.FIELDS (CONTENT-TYPE X-PRIORITY)]']['CONTENT'], $matches)) {
                       $a['MIMETYPE']=strtolower($matches[1]);
                     }
-                } elseif (isset($ret["PARSED"][$i]['EXT']['BODY[HEADER.FIELDS ("CONTENT-TYPE")]']['CONTENT'])) {
+					// fetch the priority [CONTENT] => X-Priority: 5\r\nContent-Type: multipart/alternative;\r\n\tboundary="b1_61838a67749ca51b425e42489adced98"\r\n\r\n\n
+                    if(preg_match('/x-priority: ([0-9])/iU', $ret["PARSED"][$i]['EXT']['BODY[HEADER.FIELDS (CONTENT-TYPE X-PRIORITY)]']['CONTENT'], $matches)) {
+                      $a['PRIORITY']=strtolower($matches[1]);
+                    }
+                } elseif (isset($ret["PARSED"][$i]['EXT']['BODY[HEADER.FIELDS ("CONTENT-TYPE" "X-PRIORITY")]']['CONTENT'])) {
                     // some versions of cyrus send "CONTENT-TYPE" and CONTENT-TYPE only
-                    if (preg_match('/^content-type: (.*);/iU', $ret["PARSED"][$i]['EXT']['BODY[HEADER.FIELDS ("CONTENT-TYPE")]']['CONTENT'], $matches)) {
+                    if (preg_match('/content-type: (.*);/iU', $ret["PARSED"][$i]['EXT']['BODY[HEADER.FIELDS ("CONTENT-TYPE" "X-PRIORITY")]']['CONTENT'], $matches)) {
                         $a['MIMETYPE']=strtolower($matches[1]);
+                    }
+					//  fetch the priority [CONTENT] => X-Priority: 5\r\nContent-Type: multipart/alternative;\r\n\tboundary="b1_61838a67749ca51b425e42489adced98"\r\n\r\n\n
+                    if (preg_match('/x-priority: ([0-9])/iU', $ret["PARSED"][$i]['EXT']['BODY[HEADER.FIELDS ("CONTENT-TYPE" "X-PRIORITY")]']['CONTENT'], $matches)) {
+                        $a['PRIORITY']=strtolower($matches[1]);
                     }
                 }
                 $env[]=$a;
@@ -504,18 +538,28 @@ class Net_IMAP extends Net_IMAPProtocol {
         } else {
           $ret=$this->cmdFetch($msg_id,"BODYSTRUCTURE");
         }
+		#_debug_array($ret);
         if (PEAR::isError($ret)) {
             return $ret;
         }
         if(strtoupper($ret["RESPONSE"]["CODE"]) != "OK"){
             return new PEAR_Error($ret["RESPONSE"]["CODE"] . ", " . $ret["RESPONSE"]["STR_CODE"]);
         }
-        $ret=$ret["PARSED"][0]["EXT"]["BODYSTRUCTURE"][0];
+        $ret2=$ret["PARSED"][0]["EXT"]["BODYSTRUCTURE"][0];
+		// sometimes we get an [COMMAND] => OK with $ret["PARSED"][0] and no $ret["PARSED"][0]["EXT"]["BODYSTRUCTURE"]
+		if (is_array($ret) && empty($ret2)) {
+			foreach($ret["PARSED"] as $substruct) {
+				if ($substruct["COMMAND"] == "FETCH") {
+					$ret2=$substruct["EXT"]["BODYSTRUCTURE"][0];
+					break;
+				}
+			}
+		}
         $structure = array();
 
         $mimeParts = array();
-        $this->_parseStructureArray($ret, $mimeParts);
-
+        $this->_parseStructureArray($ret2, $mimeParts);
+        #_debug_array($ret);
         return array_shift($mimeParts);
     }
 
@@ -797,10 +841,19 @@ class Net_IMAP extends Net_IMAPProtocol {
      */
     function _parseStructureCommonFields(&$_structure) 
     {
+		#error_log(__METHOD__.print_r($_structure,true)." ".function_backtrace());
         #print "Net_IMAP::_parseStructureTextArray _partID: $_partID<br>";
         $part = new stdClass;
         $part->type = strtoupper($_structure[0]);
-        $part->subType = strtoupper($_structure[1]);
+		//dovecot has no subtype for type attachment, and does not pass it as structure[2]
+        if (!is_array($_structure[1])) $part->subType = strtoupper($_structure[1]);
+		if(is_array($_structure[1])) {
+			foreach($_structure[1] as $key => $value) {
+				if($key%2 == 0) {
+					$part->parameters[strtoupper($_structure[1][$key])] = $_structure[1][$key+1];
+				}
+			}
+		}
         if(is_array($_structure[2])) {
           foreach($_structure[2] as $key => $value) {
             if($key%2 == 0) {
@@ -808,6 +861,7 @@ class Net_IMAP extends Net_IMAPProtocol {
             }
           }
         }
+        $part->filename = $_structure[4];
         $part->encoding = strtoupper($_structure[5]);
         $part->bytes = $_structure[6];
         
@@ -1218,7 +1272,7 @@ class Net_IMAP extends Net_IMAPProtocol {
      * @param   string  $mailbox        mailbox name to append to (default is current mailbox)
      * @param   string  $flags_list     set flags appended message
      *
-     * @return  mixed   true on success / Pear_Error on failure
+     * @return  mixed   true (or the uid of the created message) on success / Pear_Error on failure
      *
      * @access  public
      * @since   1.0
@@ -1232,9 +1286,16 @@ class Net_IMAP extends Net_IMAPProtocol {
         if (PEAR::isError($ret)) {
             return $ret;
         }
+
         if(strtoupper($ret["RESPONSE"]["CODE"]) != "OK"){
             return new PEAR_Error($ret["RESPONSE"]["CODE"] . ", " . $ret["RESPONSE"]["STR_CODE"]);
         }
+		// the expected response is something like that: [APPENDUID 1160024220 12] Completed
+		// the uid of the created message is the number just before the closing bracket
+		$retcode = explode(' ',$ret["RESPONSE"]["STR_CODE"]);
+		$retcode = explode(']',$retcode[2]);
+		if (intval($retcode[0]) > 0) return $retcode[0];
+		// this should never happen, exept the parsed response is not as expected
         return true;
     }
 
@@ -1332,7 +1393,7 @@ class Net_IMAP extends Net_IMAPProtocol {
      */
     function getMailboxes($reference = '', $restriction_search = 0, $returnAttributes=false)
     {
-
+		#echo (__METHOD__.$reference."#".$restriction_search.'#'.function_backtrace()."<br>");
         if ( is_bool($restriction_search) ){
             $restriction_search = (int) $restriction_search;
         }
@@ -1361,6 +1422,7 @@ class Net_IMAP extends Net_IMAPProtocol {
         if( PEAR::isError( $ret = $this->cmdList($reference, $mailbox) ) ){
             return $ret;
         }
+
         if(strtoupper($ret["RESPONSE"]["CODE"]) != "OK"){
             return new PEAR_Error($ret["RESPONSE"]["CODE"] . ", " . $ret["RESPONSE"]["STR_CODE"]);
         }
@@ -1371,7 +1433,7 @@ class Net_IMAP extends Net_IMAPProtocol {
             //If the folder has the \NoSelect atribute we don't put in the list
             // it solves a bug in wu-imap that crash the IMAP server if we select that mailbox
                 if( isset($mbox["EXT"]["LIST"]["NAME_ATTRIBUTES"]) ){
-                    #if( !in_array('\NoSelect',$mbox["EXT"]["LIST"]["NAME_ATTRIBUTES"]) ){
+                    #if( !(in_array('\NoSelect',$mbox["EXT"]["LIST"]["NAME_ATTRIBUTES"]) || in_array('\Noselect',$mbox["EXT"]["LIST"]["NAME_ATTRIBUTES"])) ){
                         if( $returnAttributes){
                             $ret_aux[]=array(   'MAILBOX' => $mbox["EXT"]["LIST"]["MAILBOX_NAME"],
                                                 'ATTRIBUTES' => $mbox["EXT"]["LIST"]["NAME_ATTRIBUTES"] ,
@@ -1401,7 +1463,7 @@ class Net_IMAP extends Net_IMAPProtocol {
     function mailboxExist($mailbox)
     {
         // true means do an exact match
-        if( PEAR::isError( $ret = $this->getMailboxes( $mailbox , true ) ) ){
+        if( PEAR::isError( $ret = $this->getMailboxes( $mailbox, 1, false ) ) ){
             return $ret;
         }
         if( count( $ret ) > 0 ){
@@ -1562,6 +1624,7 @@ class Net_IMAP extends Net_IMAPProtocol {
      */
     function listsubscribedMailboxes($reference = ''  , $restriction_search = 0, $returnAttributes = false)
     {
+		#echo __METHOD__." called for ".$reference."#$restriction_search#<br>";
         if ( is_bool($restriction_search) ){
             $restriction_search = (int) $restriction_search;
         }
@@ -1636,7 +1699,7 @@ class Net_IMAP extends Net_IMAPProtocol {
      * @access  public
      * @since   1.0
      */
-    function getFlags( $msg_id = null )
+    function getFlags( $msg_id = null , $uidStore = false)
     {
       // You can also provide an array of numbers to those emails
         if( $msg_id != null){
@@ -1648,9 +1711,13 @@ class Net_IMAP extends Net_IMAPProtocol {
         }else{
             $message_set="1:*";
         }
+		if ($uidStore == true ) {
+			$ret = $this->cmdUidFetch($message_set, 'FLAGS');
+		} else {
+			$ret = $this->cmdFetch($message_set, 'FLAGS');
+		}
 
-
-        if (PEAR::isError($ret = $this->cmdFetch($message_set, 'FLAGS'))) {
+		if (PEAR::isError($ret)) {
             return $ret;
         }
         if(strtoupper($ret["RESPONSE"]["CODE"]) != "OK"){
@@ -1686,6 +1753,7 @@ class Net_IMAP extends Net_IMAPProtocol {
      */
     function setFlags($msg_id, $flags, $mod = 'set', $uidStore = false)
     {
+		#error_log("egw-pear::Net::setFlags");
         // you can also provide an array of numbers to those emails
         if ($msg_id == 'all') {
             $message_set = '1:*';
@@ -1719,7 +1787,7 @@ class Net_IMAP extends Net_IMAPProtocol {
                 return new PEAR_Error('wrong input $mod');
                 break;
         }
-        
+        #error_log("egw-pear::Net::setFlags for Message: ".print_r($message_set,true)."->".$flaglist); 
         if($uidStore == true) {
           $ret=$this->cmdUidStore($message_set, $dataitem, $flaglist);
         } else {
